package com.huimv.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.admin.common.utils.Result;
import com.huimv.admin.common.utils.ResultCode;
import com.huimv.admin.entity.EnvDayagePeizhi;
import com.huimv.admin.entity.EnvDevice;
import com.huimv.admin.entity.EnvDeviceFan;
import com.huimv.admin.entity.EnvFengPeizhi;
import com.huimv.admin.entity.vo.PublicDeployVo;
import com.huimv.admin.mapper.EnvDayagePeizhiMapper;
import com.huimv.admin.mapper.EnvDeviceFanMapper;
import com.huimv.admin.mapper.EnvDeviceMapper;
import com.huimv.admin.mapper.EnvFengPeizhiMapper;
import com.huimv.admin.service.IEnvDayagePeizhiService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-04-10
 */
@Service
public class EnvDayagePeizhiServiceImpl extends ServiceImpl<EnvDayagePeizhiMapper, EnvDayagePeizhi> implements IEnvDayagePeizhiService {

    @Resource
    private EnvDayagePeizhiMapper envDayagePeizhiMapper;
    @Resource
    private EnvFengPeizhiMapper envFengPeizhiMapper;
    @Resource
    private EnvDeviceMapper envDeviceMapper;
    @Resource
    private EnvDeviceFanMapper envDeviceFanMapper;
    @Override
    public Result listDeploy(Map<String, String> paramsMap) {
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        Page<PublicDeployVo> page = new Page(Integer.parseInt(pageNum),Integer.parseInt(pageSize));

        IPage<PublicDeployVo> publicDeployVos = envDayagePeizhiMapper.listPublicDeploy(page);
        return new Result(ResultCode.SUCCESS,publicDeployVos);
    }

    @Override
    public Result listDeployByDevice(Map<String, String> paramsMap) {
        String deviceCode = paramsMap.get("deviceCode");
        List<EnvDayagePeizhi> envDayagePeizhis = envDayagePeizhiMapper.selectList(new QueryWrapper<EnvDayagePeizhi>()
                .eq("device_code", deviceCode).or().eq("open_status",1)
                .groupBy("time_dayage"));
        return new Result(ResultCode.SUCCESS,envDayagePeizhis);
    }

    @Override
    @Transactional
    public Result rename(Map<String, String> paramsMap) {
        String deployName = paramsMap.get("newDeployName");
        String deviceCode = paramsMap.get("deviceCode");
        String timeDayAge = paramsMap.get("timeDayAge");
        List<EnvDayagePeizhi> time_dayage = envDayagePeizhiMapper.selectList(new QueryWrapper<EnvDayagePeizhi>().eq("time_dayage", timeDayAge).eq("device_code",deviceCode));
        for (EnvDayagePeizhi envDayagePeizhi : time_dayage) {
            envDayagePeizhi.setDeployName(deployName);
            envDayagePeizhiMapper.updateById(envDayagePeizhi);
        }
        List<EnvFengPeizhi> windDeploy = envFengPeizhiMapper.selectList(new QueryWrapper<EnvFengPeizhi>().eq("time_feng", timeDayAge).eq("device_code",deviceCode));
        for (EnvFengPeizhi envWindPeizhi : windDeploy) {
            envWindPeizhi.setDeployName(deployName);
            envFengPeizhiMapper.updateById(envWindPeizhi);
        }
        return Result.SUCCESS();
    }

    @Override
    @Transactional
    public Result apply(Map<String, String> paramsMap) {
        String deviceCode = paramsMap.get("deviceCode");
        String timeDayAge = paramsMap.get("timeDayAge");
        String farmId = paramsMap.get("farmId");
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime time = LocalDateTime.parse(timeDayAge, formatter);
        System.out.println(time);
        EnvDevice envDevice = envDeviceMapper.selectOne(new QueryWrapper<EnvDevice>().eq("device_sort_code", deviceCode).eq("farm_id", farmId));
        envDevice.setDisposeTime(time);
        List<EnvFengPeizhi> timeDayAge1 = envFengPeizhiMapper.selectList(new QueryWrapper<EnvFengPeizhi>().eq("time_feng",timeDayAge));
        for (EnvFengPeizhi envFengPeizhi : timeDayAge1) {
            String fansCtrl = envFengPeizhi.getFansCtrl();
            Integer fanNum = envDeviceFanMapper.selectCount(new QueryWrapper<EnvDeviceFan>().eq("device_code", deviceCode).eq("farm_id", farmId));
            if (fanNum != fansCtrl.length()){
                return new Result(ResultCode.FAIL,"设备风机数量不等于配置所需！");
            }
        }
        envDeviceMapper.updateById(envDevice);
        //应用的变1
        List<EnvDayagePeizhi> envDayagePeizhis = envDayagePeizhiMapper.selectList(new QueryWrapper<EnvDayagePeizhi>().eq("device_code", deviceCode).eq("time_dayage", time));
        for (EnvDayagePeizhi envDayagePeizhi : envDayagePeizhis) {
            envDayagePeizhi.setUsingStart(1);
            envDayagePeizhiMapper.updateById(envDayagePeizhi);
        }
        List<EnvFengPeizhi> envFengPeizhis = envFengPeizhiMapper.selectList(new QueryWrapper<EnvFengPeizhi>().eq("device_code", deviceCode).eq("time_feng", time));
        for (EnvFengPeizhi envFengPeizhi : envFengPeizhis) {
            envFengPeizhi.setUsingStart(1);
            envFengPeizhiMapper.updateById(envFengPeizhi);
        }
        //其他0
        List<EnvDayagePeizhi> envDayage = envDayagePeizhiMapper.selectList(new QueryWrapper<EnvDayagePeizhi>().eq("device_code", deviceCode).ne("time_dayage", time));
        for (EnvDayagePeizhi envDayagePeizhi : envDayage) {
            envDayagePeizhi.setUsingStart(0);
            envDayagePeizhiMapper.updateById(envDayagePeizhi);
        }
        List<EnvFengPeizhi> envFeng = envFengPeizhiMapper.selectList(new QueryWrapper<EnvFengPeizhi>().eq("device_code", deviceCode).ne("time_feng", time));
        for (EnvFengPeizhi envFengPeizhi : envFeng) {
            envFengPeizhi.setUsingStart(0);
            envFengPeizhiMapper.updateById(envFengPeizhi);
        }
        return Result.SUCCESS();
    }

    @Override
    @Transactional
    public Result uplode(Map<String, String> paramsMap) {
        String timeDayAge = paramsMap.get("timeDayAge");
        String deviceCode = paramsMap.get("deviceCode");
        List<EnvDayagePeizhi> envDayagePeizhis = envDayagePeizhiMapper.selectList(new QueryWrapper<EnvDayagePeizhi>().eq("device_code", deviceCode).eq("time_dayage", timeDayAge));
        for (EnvDayagePeizhi envDayagePeizhi : envDayagePeizhis) {
            envDayagePeizhi.setOpenStatus(1);
            envDayagePeizhiMapper.updateById(envDayagePeizhi);
        }
        List<EnvFengPeizhi> envFengPeizhis = envFengPeizhiMapper.selectList(new QueryWrapper<EnvFengPeizhi>().eq("device_code", deviceCode).eq("time_feng", timeDayAge));
        for (EnvFengPeizhi envFengPeizhi : envFengPeizhis) {
            envFengPeizhi.setOpenStatus(1);
            envFengPeizhiMapper.updateById(envFengPeizhi);
        }
        return Result.SUCCESS();
    }

    @Override
    public Result listDayAge(Map<String, String> paramsMap) {
        String deviceCode = paramsMap.get("deviceCode");
        String timeDayAge = paramsMap.get("timeDayAge");
        List<EnvDayagePeizhi> envDayagePeizhis = envDayagePeizhiMapper.selectList(new QueryWrapper<EnvDayagePeizhi>()
                .eq("device_code", deviceCode)
                .eq("time_dayage", timeDayAge)
                .orderByAsc("level_dayage"));
        return new Result(ResultCode.SUCCESS,envDayagePeizhis);
    }


}
